{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT DMA;
{ Unit that handles Direct Memory Access.                                  }
{ Created          : 13/05/98                                              }
{ Last modification: 23/05/98                                              }

{{$DEFINE DEBUGINFO}

INTERFACE

CONST
     {ports}
     _8bitMask          = $0A;
     _16bitMask         = $D4;
     _8bitFlipFlop      = $0C;
     _16bitFlipFlop     = $D8;
     _8bitType          = $0B;
     _16bitType         = $D6;
     {commands}
     SinglePlay         = $48;
     SingleRecord       = $44;
     AutoInitPlay       = $58;
     AutoInitRecord     = $54;

{==========================================================================}
Procedure AllocateBuffer(var buffer: pointer; size: dword);
{allocates a page aligned (or rather 128kb) piece of memory. If not        }
{succesfull buffer = 0.                                                    }
Procedure DisableChannel(Channel: byte);
{disables the given DMA channel. This has to be done before you can program}
{the channel.                                                              }
Procedure EnableChannel(Channel: byte);
{enables the given DMA channel. This has to be done after the programming. }
Procedure ResetFlipFlop(Channel: byte);
{Resets the Flip/Flop for the DMA channel.                                 }
Procedure ProgramChannel(Channel,DMAType: byte; Address: pointer; Length: word);
{Programs the DMA channel. See constants for DMAtype.                      }
{==========================================================================}

IMPLEMENTATION

USES
     DPMILIB{,DB},STRINGS;

{==========================================================================}
Procedure AllocateBuffer(var buffer: pointer; size: dword);
 var address,segment,newsize: dword;
 begin
  {$IFDEF DEBUGINFO}
   DB.Writestring('DMA.Alloc Buf : ');
  {$ENDIF}
  dword(buffer):=0;
  segment:=DosMemoryAlloc(size);
  if segment=0 then
   begin
    {$IFDEF DEBUGINFO}
     Db.Writelnstring('Failed.');
    {$ENDIF}
    exit;
   end;
  address:=segment shl 4;
  if ((address+size-1) and $1FFFF)<(address and $1FFFF) then
   begin
    DosMemoryFree(segment);
    newsize:=size+((address+size-1) and $F0000)-address;
    {$IFDEF DEBUGINFO}
     Db.Writelnstring('Not page aligned. Allocating '+hex(newsize)+'h bytes instead');
     Db.Writestring('DMA.Alloc Buf : ');
    {$ENDIF}
    segment:=DosMemoryAlloc(word(newsize));
    if segment=0 then
     begin
      {$IFDEF DEBUGINFO}
       Db.Writelnstring('Failed.');
      {$ENDIF}
      exit;
     end;
    address:=((segment shl 4)+size) and $F0000;
   end;
   {$IFDEF DEBUGINFO}
    DB.Writeword(word(size));
    DB.WriteString(' bytes at address ');
    DB.WriteDword(Address);
    DB.Writelnstring('.');
   {$ENDIF}
   dword(buffer):=address;
 end;
{==========================================================================}
Procedure DisableChannel(Channel: byte);
 begin
  {$IFDEF DEBUGINFO}
   DB.WriteString('DMA.Disable   : Channel ');
   DB.WriteByte(Channel);
   DB.WritelnString('.');
  {$ENDIF}
  case channel of
   0..3 : port[_8bitMask]:=4+Channel;
   4..7 : port[_16bitMask]:=Channel;
  end;
 end;
{==========================================================================}
Procedure EnableChannel(Channel: byte);
 begin
  {$IFDEF DEBUGINFO}
   DB.WriteString('DMA.Enable    : Channel ');
   DB.WriteByte(Channel);
   DB.WritelnString('.');
  {$ENDIF}
  case channel of
  0..3 : port[_8bitMask]:=Channel;
  4..7 : port[_16bitMask]:=Channel and 3;
  end;
 end;
{==========================================================================}
Procedure ResetFlipFlop(Channel: byte);
 begin
  {$IFDEF DEBUGINFO}
   DB.WriteString('DMA.Reset F/F : Channel ');
   DB.WriteByte(Channel);
   DB.WritelnString('.');
  {$ENDIF}
  case channel of
  0..3 : port[_8bitFlipFlop]:=0;
  4..7 : port[_16bitFlipFlop]:=0;
  end;
 end;
{==========================================================================}
Procedure ProgramChannel(Channel,DMAType: byte; Address: pointer; Length: word);
 const PageRegister: array[0..7] of byte = ($87,$83,$81,$82,$8F,$8B,$89,$8A);
 begin
  DMA.ResetFlipFlop(Channel);
  case Channel of
  0..3 : Port[_8bitType]:=DMAType+Channel;
  4..7 : Port[_16bitType]:=DMAType+(Channel and 3);
  end;
  {$IFDEF DEBUGINFO}
   DB.WriteString('DMA.Type      : ');
   DB.WriteByte(DMAType);
   DB.WritelnString('.');
  {$ENDIF}
  case Channel of
  0..3 : begin
          Port[Channel shl 1]:=byte(Address); {lo(address)}
          Port[Channel shl 1]:=byte(dword(Address) shr 8); {hi(address)}
          Port[(Channel shl 1)+1]:=byte(Length-1); {lo(length-1)}
          Port[(Channel shl 1)+1]:=byte((Length-1) shr 8); {hi(length-1)}
         end;
  4..7 : begin
          dword(address):=dword(address) shr 1; {address has to be divided by 2 for 16bit}
          Port[$B0+(Channel shl 2)]:=byte(Address); {lo(address)}
          Port[$B0+(Channel shl 2)]:=byte(dword(Address) shr 8); {hi(address)}
          Port[$B2+(Channel shl 2)]:=byte(Length-1); {lo(length-1)}
          Port[$B2+(Channel shl 2)]:=byte((Length-1) shr 8); {hi(length-1)}
         end;
  end;
  Port[Pageregister[Channel]]:=byte(dword(address) shr 16); {page}
 end;
BEGIN
END.