{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT DSP;
{ Unit that handles the DSP chip of the Soundblaster 16.                   }
{ Created          : 11/05/98                                              }
{ Last modification: 22/05/98                                              }

{{$DEFINE DEBUGINFO}

INTERFACE

CONST
     {ports}
     Reset_cmd          = $06;
     Read_cmd           = $0A;
     Write_cmd          = $0C;
     Data_avail         = $0E;
     _8bitIRQack        = $0E;
     _16bitIRQack       = $0F;
     {DSP commands}
     Set_Smplrate       = $41;
     Pause8bit          = $D0;
     Continue8bit       = $D4;
     Pause16bit         = $D5;
     Continue16bit      = $D6;
     Exit8bit           = $DA;
     Exit16bit          = $D9;
     Enable_spk         = $D1;
     Disable_spk        = $D3;
     Get_Version        = $E1;
     Get_Copyright      = $E3;
     SineGenerate       = $F0;
     Trig8bitIrq        = $F2;
     Trig16bitIrq       = $F3;
     Get_Status_cmd     = $FB;
     Get_AuxStatus      = $FC;
     {DSP types}
     Fifo               = $02;
     Auto_Init          = $04;
     DAC                = $00;
     ADC                = $08;
     _8bit              = $C0;
     _16bit             = $B0;
     {DSP modes}
     Signed             = $10;
     Unsigned           = $00;
     Mono               = $00;
     Stereo             = $20;

{==========================================================================}
Procedure WriteCmd(Command: dword);
{==========================================================================}
Function  ReadCmd: byte;
{==========================================================================}
Function  Reset: boolean;
{==========================================================================}
Procedure Set_Samplerate(Samplerate: word);
{==========================================================================}
Procedure SetupDMAmode(DSP_Type,DSP_Mode: byte; length: word);
{==========================================================================}
Procedure Exit8bitDMA;
{==========================================================================}
Procedure Exit16bitDMA;
{==========================================================================}
Procedure Pause8bitDMA;
{==========================================================================}
Procedure Pause16bitDMA;
{==========================================================================}
Function  GetVersion: word;
{==========================================================================}
Procedure SendEndOfInterrupt(DMAChannel: byte);
{==========================================================================}

IMPLEMENTATION

USES VARIABLE{,DB};

{==========================================================================}
Procedure WriteCmd(Command: dword);
 begin
  repeat
  until (port[BaseAddress+Write_Cmd]<128);
                                          {wait until bit 7 is cleared}
  port[baseAddress+Write_Cmd]:=Command;
 end;
{==========================================================================}
Function ReadCmd: byte;
 begin
  repeat
  until (port[BaseAddress+Data_avail]>=128);
                                          {wait until bit 7 is set}
  ReadCmd:=port[BaseAddress+Read_Cmd];
 end;
{==========================================================================}
Function Reset: boolean;
 var a: dword;
 begin
  Reset:=False;
  {$IFDEF DEBUGINFO}
   DB.WriteString('DSP.Reset     : ');
  {$ENDIF}

  Port[BaseAddress+Reset_Cmd]:=1;
  for a:=1 to 10000 do begin end;         {wait 3 usecs}
  port[BaseAddress+Reset_Cmd]:=0;
  repeat
   dec(a);
   if a=0 then
    begin
     {$IFDEF DEBUGINFO}
      DB.Writelnstring('FAILED.');
     {$ENDIF}
     exit;                                {not set? - return false}
    end;
  until (Port[baseAddress+Data_Avail]>=128);
                                          {wait until DSP_data_avail bit 7 is set}
  repeat
   inc(a);
   if a=100000 then exit;                 {not AA (in 100 usecs) - return false}
  until (port[baseaddress+Read_cmd]=$AA);

  {$IFDEF DEBUGINFO}
   DB.WritelnString('Succesfull.');
  {$ENDIF}
  Reset:=True;
 end;
{==========================================================================}
Procedure Set_Samplerate(Samplerate: word);
 begin
  {$IFDEF DEBUGINFO}
   DB.WriteString('DSP.Samplerate: ');
   DB.WriteWord(samplerate);
   DB.WritelnString('hz.');
  {$ENDIF}
  DSP.WriteCmd(Set_Smplrate);
  DSP.WriteCmd(hi(Samplerate));
  DSP.WriteCmd(lo(samplerate));
 end;
{==========================================================================}
Procedure SetupDMAmode(DSP_Type,DSP_Mode: byte; length: word);
 begin
  DSP.WriteCmd(DSP_Type);
  DSP.WriteCmd(DSP_Mode);
  DSP.WriteCmd(byte(Length-1));
  DSP.WriteCmd(byte((Length-1) shr 8));
 end;
{==========================================================================}
Procedure Exit8bitDMA;
 begin
  DSP.WriteCmd(Exit8bit);
  {$IFDEF DEBUGINFO}
   DB.WritelnString('DSP.Exit 8bit : Succesfull');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Exit16bitDMA;
 begin
  DSP.WriteCmd(Exit16bit);
  {$IFDEF DEBUGINFO}
   DB.WritelnString('DSP.Exit 16bit: Succesfull');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Pause8bitDMA;
 begin
  DSP.WriteCmd(Pause8bit);
  {$IFDEF DEBUGINFO}
   DB.WritelnString('DSP.Pause 8bit: Succesfull');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Pause16bitDMA;
 begin
  DSP.WriteCmd(Pause16bit);
  {$IFDEF DEBUGINFO}
   DB.WritelnString('DSP.Pause16bit: Succesfull');
  {$ENDIF}
 end;
{==========================================================================}
Function GetVersion: word;
 var a: dword;
 {$IFDEF DEBUGINFO}
 var Copyright: string;
 {$ENDIF}
 begin
  DSP.WriteCmd(Get_version);
  a:=DSP.ReadCmd shl 8 + DSP.ReadCmd;

  GetVersion:=a;

  {$IFDEF DEBUGINFO}
   DB.WriteString('DSP.Version   : ');
   DB.WriteWord(word(a));
   DB.WritelnString('.');

   copyright:='';
   a:=0;
   DSP.WriteCmd(Get_Copyright);
   repeat
    copyright:=copyright+char(DSP.ReadCmd);
    inc(a);
   until copyright[a]=#0;
   DB.WritelnString('DSP.Copyright : '+copyright);
  {$ENDIF}
 end;
{==========================================================================}
Procedure SendEndOfInterrupt(DMAChannel: byte);
 begin
  if DMAChannel=DMA8 then Port[baseaddress+_8bitIRQack]
  else Port[baseaddress+_16bitIRQack];
 end;
{==========================================================================}

BEGIN
END.