{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT EQ;
{ Unit that does the EQ.                                                   }
{ Created          : 14/05/98                                              }
{ Last modification: 24/05/98                                              }

INTERFACE

{{$DEFINE DEBUGINFO}

{==========================================================================}
Procedure Input_16_to_FP;
{waits for input and coverts it to F.P. in X1 and X2.                      }
Procedure Calculate_FirFilter;
{Calculates the final FIR filter.                                          }
Procedure Do_FirFilter;
{does the FIR filter on X1+X2 and puts the output in Y1 and Y2.            }
Procedure Do_Gate;
{does the gate function on Y1 and Y2                                       }
Procedure CalculateVUPos;
{finds the maximum peak in the buffers and updates the GUI.                }
Procedure Do_RasmusRadiomus;
{changes the pitch of the channel to 2x                                    }
Procedure SetVolume;
{Sets the volume for both Channels. (Y*Volume*crossvolume*master)          }
Procedure Output_Mix_FP_to_8;
{Mixes the channels and puts the result in the outputbuffer.               }
{==========================================================================}

IMPLEMENTATION

USES VARIABLE,GUI2{,DB};

{==========================================================================}
Procedure Input_16_to_FP;
 var n: dword;
 begin
  repeat until OldInputPosition<>InputPosition;
  {wait until new data is ready in buffer.}
  OldInputPosition:=InputPosition;
  {update OldInputPosition}
  for n:=0 to BufferSize-1 do
   begin
    X1[n+OldInputPosition]:=InputBuffer^[n+OldInputPosition,1];
   end;
  for n:=0 to BufferSize-1 do
   begin
    X2[n+OldInputPosition]:=InputBuffer^[n+OldInputPosition,2];
   end;
  {convert to F.P}
 end;
{====================================================================}
Procedure Calculate_FIRFilter;
 var n: dword;
 begin
  if RecalcImpulse1=True then
   begin
    {only calculate if EQ 1 changed}
    for n:=0 to FirOrder do
     begin
      H1[n]:=(A1lowpass*Hlowpass[n]+A1bandpass*Hbandpass[n]+A1highpass*Hhighpass[n])*Gain1;
     end;
    {$IFDEF DEBUGINFO}
     DB.Writelnstring('EQ.Calculate  : 1');
    {$ENDIF}
    RecalcImpulse1:=False;
   end;

  if RecalcImpulse2=True then
   begin
    {only calculate if EQ 2 changed}
    for n:=0 to FirOrder do
     begin
     H2[n]:=(A2lowpass*Hlowpass[n]+A2bandpass*Hbandpass[n]+A2highpass*Hhighpass[n])*Gain2;
     end;
    ReCalcImpulse2:=False;
    {$IFDEF DEBUGINFO}
     DB.Writelnstring('EQ.Calculate  : 2');
    {$ENDIF}
  end;
 end;
{====================================================================}
Procedure Do_FirFilter;
 var i,n: longint;
 begin
  asm
   {eax=offset in X1}
   {ecx=counter (which sample in Y1 is currently calculated)}
   {edx=offset in H1}
   fninit;
   {channel1}
   mov eax,[OldInputposition]
   dec eax
   xor ecx,ecx

  @nextsample1:
   sub eax,(FirOrder-1)
   xor edx,edx
   and eax,(2*Buffersize-1)

   fld qword [X1+eax*8] {st0=X1[eax]}
   fmul qword [H1+edx*8] {st0=X1[eax]*H1[0]}

   inc eax
   inc edx
   and eax,(2*Buffersize-1)

  @nexttap1:
   fld qword [X1+eax*8] {st0=X1[eax],st1=Y1[n]}
   inc eax
   fmul qword [H1+edx*8]{st0=X1[eax]*H1,st1=Y1[n]}
   and eax,(buffersize*2-1)
   inc edx
   faddp                {st0=st0+st1}
   cmp edx,(FirOrder)
   jne @nexttap1

   fstp qword [Y1+ecx*8];

   inc ecx
   cmp ecx,(Buffersize)
   jne @nextsample1

   {channel2}
   mov eax,[OldInputposition]
   dec eax
   xor ecx,ecx

  @nextsample:
   sub eax,(FirOrder-1)
   xor edx,edx
   and eax,(2*Buffersize-1)

   fld qword [X2+eax*8] {st0=X2[eax]}
   fmul qword [H2+edx*8] {st0=X2[eax]*H2[0]}

   inc eax
   inc edx
   and eax,(2*Buffersize-1)

  @nexttap:
   fld qword [X2+eax*8] {st0=X2[eax],st2=Y2[n]}
   inc eax
   fmul qword [H2+edx*8]{st0=X2[eax]*H2,st2=Y2[n]}
   and eax,(buffersize*2-1)
   inc edx
   faddp                {st0=st0+st1}
   cmp edx,(FirOrder)
   jne @nexttap

   fstp qword [Y2+ecx*8];

   inc ecx
   cmp ecx,(Buffersize)
   jne @nextsample
  end;
 end;
{====================================================================}
Procedure Do_Gate;
 var n: longint;
 begin
  for n:=0 to BufferSize-1 do
   begin
    if Gate1HoldCount<Gate1HoldTime then
     begin
      Gate1Amp:=Gate1Amp+Gate1AttackSpeed;
      if Gate1Amp>1 then Gate1Amp:=1;
      if Abs(Y1[n])<Gate1Threshold then
       Gate1HoldCount:=Gate1HoldCount+1
      else
       Gate1HoldCount:=0;
     end
    else
     begin
      Gate1Amp:=Gate1Amp-Gate1ReleaseSpeed;
      if Gate1Amp<0 then Gate1Amp:=0;
      Gate1HoldCount:=88200;
     end;

    if Abs(Y1[n])>(Gate1Threshold+Gate1Hysterisis) then Gate1HoldCount:=0;

    Y1[n]:=Y1[n]*Gate1Amp;
   end;

 for n:=0 to BufferSize-1 do
  begin
   if Gate2HoldCount<Gate2HoldTime then
    begin
     Gate2Amp:=Gate2Amp+Gate2AttackSpeed;
     if Gate2Amp>1 then Gate2Amp:=1;
     if Abs(Y2[n])<Gate2Threshold then
      Gate2HoldCount:=Gate2HoldCount+1
     else
      Gate2HoldCount:=0;
    end
   else
    begin
     Gate2Amp:=Gate2Amp-Gate2ReleaseSpeed;
     if Gate2Amp<0 then Gate2Amp:=0;
     Gate2HoldCount:=88200;
    end;

   if Abs(Y2[n])>(Gate2Threshold+Gate2Hysterisis) then Gate2HoldCount:=0;

   Y2[n]:=Y2[n]*Gate2Amp;
  end;
 end;
{====================================================================}
Procedure CalculateVUPos;
 var n: dword;
     Pos1, Pos2: double;
     VU1,VU2: dword;
 Begin
  Pos1:=0;
  for n:=0 to BufferSize-1 do If Abs(Y1[n])>Pos1 then Pos1:=Abs(Y1[n]);
  {find biggest (abs) value}
  VU1:=round(Pos1*(256/32768)*Volume1);
  {convert to integer}
  if VU1>255 then VU1:=255;
  {clip}
  Pos2:=0;
  for n:=0 to BufferSize-1 do If Abs(Y2[n])>Pos2 then Pos2:=Abs(Y2[n]);
  {find biggest (abs) value}
  VU2:=round(Pos2*(256/32768)*Volume2);
  {convert to integer}
  if VU2>255 then VU2:=255;
  {clip}
  GUI2.UpdateVUs(byte(VU1),Byte(VU2))
  {update GUI}
 End;
{====================================================================}
Procedure Do_RasmusRadiomus;
 var n: dword;
 begin
  if DoRasmusRadiomus1 = true then
   begin 
    for n:=0 to (Buffersize shr 1) do
     begin
      Y1[n]:=Y1[n shl 1];
     end;
    for n:=((Buffersize shr 1)+1) to Buffersize do
     begin
      Y1[n]:=Y1[n-(Buffersize shr 1)];
     end;
   end;

  if DoRasmusRadiomus2 = true then
   begin 
    for n:=0 to (Buffersize shr 1) do
     begin
      Y2[n]:=Y2[n shl 1];
     end;
    for n:=((Buffersize shr 1)+1) to Buffersize do
     begin
      Y2[n]:=Y2[n-(Buffersize shr 1)];
     end;
   end;
 end;
{====================================================================}
Procedure SetVolume;
 var n: dword;
     Temp1,Temp2: double;
 begin
  if CrossVolume>1 then
   begin
    Temp1:=Volume1*(2-CrossVolume)*MasterVolume;
    Temp2:=Volume2*MasterVolume;
   end
  else
   begin
    Temp1:=Volume1*MasterVolume;
    Temp2:=Volume2*CrossVolume*MasterVolume;
   end;

  for n:=0 to BufferSize-1 do
   begin
    Y1[n]:=Y1[n]*Temp1;
   end;

  for n:=0 to BufferSize-1 do
   begin
    Y2[n]:=Y2[n]*Temp2;
   end;
 end;
{====================================================================}
Procedure Output_Mix_FP_to_8;
 var a,n: longint;
     Clipped: boolean;
 begin
  Clipped:=False;
  Repeat Until OldOutputPosition<>OutputPosition;
  {wait until new data is ready in buffer.}
  OldoutputPosition:=OutputPosition;
  {update OldInputPosition}
  for n:=0 to BufferSize-1 do
   begin
    a:=round((Y1[n]+Y2[n])*(128/(32768*2)));
    {mix}
    if a<-128 then
     begin
      {clip}
      a:=-128;
      Clipped:=True;
     end;
    if a>127 then
     begin
      {clip}
      a:=127;
      Clipped:=True;
     end;
    Outputbuffer^[OldOutputposition+n]:=shortint(a);
   end;
  GUI2.UpdateClippedLight(Clipped);
  {Update Clip indicator}
 end;
{====================================================================}

BEGIN
END.