{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT MIXER;
{ Unit that handles the MIXER chip of the Soundblaster 16.                 }
{ Created          : 11/05/98                                              }
{ Last modification: 22/05/98                                              }

{{$DEFINE DEBUGINFO}

INTERFACE
{==========================================================================}
Procedure Reset;
{==========================================================================}
Procedure Set_Master_Volume(Right_Volume,Left_Volume: byte);
{==========================================================================}
Procedure Set_DAC_Volume(Right_Volume,Left_Volume: byte);
{==========================================================================}
Procedure Set_FM_Volume(Right_Volume,Left_Volume: byte);
{==========================================================================}
Procedure Set_CDaudio_Volume(Right_Volume,Left_Volume: byte);
{==========================================================================}
Procedure Set_LineIn_Volume(Right_Volume,Left_Volume: byte);
{==========================================================================}
Procedure Set_Mic_Volume(Volume: byte);
{==========================================================================}
Procedure Set_PCspeaker_Volume(Volume: byte);
{==========================================================================}
Procedure Set_Output_Control(Microphone,Right_CDaudio,
          Left_CDaudio,Right_LineIn,Left_LineIn: boolean);
{==========================================================================}
Procedure Set_Input_Control_Right(Microphone,Right_CDaudio,Left_CDaudio,
          Right_LineIn,Left_LineIn,Right_FM,Left_FM: boolean);
{==========================================================================}
Procedure Set_Input_Control_Left(Microphone,Right_CDaudio,Left_CDaudio,
          Right_LineIn,Left_LineIn,Right_FM,Left_FM: boolean);
{==========================================================================}
Procedure Set_Input_Gain(Right,Left: byte);
{==========================================================================}
Procedure Set_Output_Gain(Right,Left: byte);
{==========================================================================}
Procedure Set_AutoGainCtrl(Value: boolean);
{==========================================================================}
Procedure Set_Treble(Right,Left: byte);
{==========================================================================}
Procedure Set_Bass(Right,Left: byte);
{==========================================================================}
Function Get_IRQ: byte;
{==========================================================================}
Function Get_DMA: byte;
{==========================================================================}
Function Get_IRQ_Status: byte;
{==========================================================================}

IMPLEMENTATION

USES VARIABLE{,DB};

CONST
     {ports}
     Register_Port      = $04;
     Data_Port          = $05;
     {commands}
     Reset_cmd          = $00;
     Master_left        = $30;
     Master_right       = $31;
     DAC_left           = $32;
     DAC_right          = $33;
     FM_left            = $34;
     FM_right           = $35;
     CDaudio_left       = $36;
     CDaudio_right      = $37;
     LineIn_left        = $38;
     LineIn_Right       = $39;
     Microphone         = $3A;
     PCspeaker          = $3B;
     Output_ctrl        = $3C;
     Input_ctrl_L       = $3D;
     Input_ctrl_R       = $3E;
     Input_Gain_L       = $3F;
     Input_Gain_R       = $40;
     Output_Gain_L      = $41;
     Output_Gain_R      = $42;
     AutoGainCtrl       = $43;
     Treble_Left        = $44;
     Treble_Right       = $45;
     Bass_Left          = $46;
     Bass_Right         = $47;
     IRQ_Info           = $80;
     DMA_Info           = $81;
     IRQ_Status         = $82;
{==========================================================================}
Procedure Reset;
 var a: dword;
 begin
  port[BaseAddress+Register_Port]:=Reset_cmd;
  for a:=0 to 100000 do begin end;
  port[BaseAddress+Data_Port]:=Reset_cmd;
  {$IFDEF DEBUGINFO}
  Db.Writelnstring('MIXER.Reset   : succesfull.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Master_Volume(Right_Volume,Left_Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=Master_left;
  port[BaseAddress+Data_Port]:=Left_Volume and $F8;
  port[BaseAddress+Register_Port]:=Master_right;
  port[BaseAddress+Data_Port]:=Right_Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.Master  : L:');
   Db.writebyte(Left_Volume);
   Db.writestring(' - R:');
   Db.writebyte(Right_Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_DAC_Volume(Right_Volume,Left_Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=DAC_left;
  port[BaseAddress+Data_Port]:=Left_Volume and $F8;
  port[BaseAddress+Register_Port]:=DAC_right;
  port[BaseAddress+Data_Port]:=Right_Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.DAC     : L:');
   Db.writebyte(Left_Volume);
   Db.writestring(' - R:');
   Db.writebyte(Right_Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_FM_Volume(Right_Volume,Left_Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=FM_left;
  port[BaseAddress+Data_Port]:=Left_Volume and $F8;
  port[BaseAddress+Register_Port]:=FM_right;
  port[BaseAddress+Data_Port]:=Right_Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.FM      : L:');
   Db.writebyte(Left_Volume);
   Db.writestring(' - R:');
   Db.writebyte(Right_Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_CDaudio_Volume(Right_Volume,Left_Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=CDaudio_left;
  port[BaseAddress+Data_Port]:=Left_Volume and $F8;
  port[BaseAddress+Register_Port]:=CDaudio_right;
  port[BaseAddress+Data_Port]:=Right_Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.CDaudio : L:');
   Db.writebyte(Left_Volume);
   Db.writestring(' - R:');
   Db.writebyte(Right_Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_LineIn_Volume(Right_Volume,Left_Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=LineIn_left;
  port[BaseAddress+Data_Port]:=Left_Volume and $F8;
  port[BaseAddress+Register_Port]:=LineIn_right;
  port[BaseAddress+Data_Port]:=Right_Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.LineIn  : L:');
   Db.writebyte(Left_Volume);
   Db.writestring(' - R:');
   Db.writebyte(Right_Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Mic_Volume(Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=Microphone;
  port[BaseAddress+Data_Port]:=Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.Mic     : ');
   Db.writebyte(Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_PCspeaker_Volume(Volume: byte);
 begin
  port[BaseAddress+Register_Port]:=PCspeaker;
  port[BaseAddress+Data_Port]:=Volume and $F8;

  {$IFDEF DEBUGINFO}
   Db.writestring('MIXER.PCspkr  : ');
   Db.writebyte(Volume);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Output_Control(Microphone,Right_CDaudio,
          Left_CDaudio,Right_LineIn,Left_LineIn: boolean);
 begin
  Port[BaseAddress+Register_Port]:=Output_Ctrl;
  Port[BaseAddress+Data_Port]:=byte(Microphone)+(byte(Right_CDaudio) shl 1)
  +(byte(Left_CDaudio) shl 2)+(byte(Right_LineIn) shl 3)+
  (byte(Left_LineIn) shl 4);
  {$IFDEF DEBUGINFO}
   Db.Writelnstring('MIXER.Output  : Micro R_CDa L_CDa R_Lin L_Lin');
   Db.Writestring('                 ');
   IF Microphone then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_LineIn then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_LineIn then Db.Writelnstring('ON') else Db.Writelnstring('OF');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Input_Control_Right(Microphone,Right_CDaudio,Left_CDaudio,
          Right_LineIn,Left_LineIn,Right_FM,Left_FM: boolean);
 begin
  Port[BaseAddress+Register_Port]:=Input_Ctrl_R;
  Port[BaseAddress+Data_Port]:=byte(Microphone)+(byte(Right_CDaudio) shl 1)
  +(byte(Left_CDaudio) shl 2)+(byte(Right_LineIn) shl 3)+
  (byte(Left_LineIn) shl 4)+(byte(Right_FM) shl 5)+(byte(Left_FM) shl 6);

  {$IFDEF DEBUGINFO}
   Db.Writelnstring('MIXER.Input R : Micro R_CDa L_CDa R_Lin L_Lin R_FM  L_FM');
   Db.Writestring('                 ');
   IF Microphone then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_LineIn then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_LineIn then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_FM then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_FM then Db.Writelnstring('ON') else Db.Writelnstring('OF');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Input_Control_Left(Microphone,Right_CDaudio,Left_CDaudio,
          Right_LineIn,Left_LineIn,Right_FM,Left_FM: boolean);
 begin
  Port[BaseAddress+Register_Port]:=Input_Ctrl_L;
  Port[BaseAddress+Data_Port]:=byte(Microphone)+(byte(Right_CDaudio) shl 1)
  +(byte(Left_CDaudio) shl 2)+(byte(Right_LineIn) shl 3)+
  (byte(Left_LineIn) shl 4)+(byte(Right_FM) shl 5)+(byte(Left_FM) shl 6);

  {$IFDEF DEBUGINFO}
   Db.Writelnstring('MIXER.Input L : Micro R_CDa L_CDa R_Lin L_Lin R_FM  L_FM');
   Db.Writestring('                 ');
   IF Microphone then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_CDaudio then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_LineIn then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_LineIn then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Right_FM then Db.Writestring('ON    ') else Db.Writestring('OFF   ');
   IF Left_FM then Db.Writelnstring('ON') else Db.Writelnstring('OF');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Input_Gain(Right,Left: byte);
 begin
  Port[BaseAddress+Register_Port]:=Input_Gain_R;
  Port[BaseAddress+Data_Port]:=Right shl 6;
  Port[BaseAddress+Register_Port]:=Input_Gain_L;
  Port[BaseAddress+Data_Port]:=Right shl 6;
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.In_Gain : L:');
   Db.Writebyte(Left);
   Db.Writestring(' - R:');
   Db.Writebyte(Right);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Output_Gain(Right,Left: byte);
 begin
  Port[BaseAddress+Register_Port]:=Output_Gain_R;
  Port[BaseAddress+Data_Port]:=Right shl 6;
  Port[BaseAddress+Register_Port]:=Output_Gain_L;
  Port[BaseAddress+Data_Port]:=Right shl 6;
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.Out_Gain: L:');
   Db.Writebyte(Left);
   Db.Writestring(' - R:');
   Db.Writebyte(Right);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_AutoGainCtrl(Value: boolean);
 begin
  Port[BaseAddress+Register_Port]:=AutoGainCtrl;
  Port[BaseAddress+Data_Port]:=byte(Value);
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.AutoGain: ');
   if value=true then Db.Writelnstring('Enabled.') else
    Db.Writelnstring('DISABLED.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Treble(Right,Left: byte);
 begin
  Port[BaseAddress+Register_Port]:=Treble_Right;
  Port[BaseAddress+Data_Port]:=Right shl 4;
  Port[BaseAddress+Register_Port]:=Treble_Left;
  Port[BaseAddress+Data_Port]:=Left shl 4;
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.Treble  : L:');
   Db.Writebyte(Left);
   Db.Writestring(' - R:');
   Db.Writebyte(Right);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Procedure Set_Bass(Right,Left: byte);
 begin
  Port[BaseAddress+Register_Port]:=Bass_Right;
  Port[BaseAddress+Data_Port]:=Right shl 4;
  Port[BaseAddress+Register_Port]:=Bass_Left;
  Port[BaseAddress+Data_Port]:=Left shl 4;
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.Bass    : L:');
   Db.Writebyte(Left);
   Db.Writestring(' - R:');
   Db.Writebyte(Right);
   Db.Writelnstring('.');
  {$ENDIF}
 end;
{==========================================================================}
Function Get_IRQ: byte;
 var a: byte;
 begin
  Port[BaseAddress+Register_Port]:=Irq_Info;
  a:=Port[BaseAddress+Data_Port] and $F;
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.IRQ     : ');
   case a of
    1: Db.Writelnstring('2.');
    2: Db.Writelnstring('5.');
    4: Db.Writelnstring('7.');
    8: Db.Writelnstring('10.');
   else Db.Writelnstring('Multiple.'); end;
  {$ENDIF}
   case a of
    1: Get_IRQ:=2;
    2: Get_IRQ:=5;
    4: Get_IRQ:=7;
    8: Get_IRQ:=10;
   else
    Get_IRQ:=$FF;
   end;
 end;
{==========================================================================}
Function Get_DMA: byte;
 var a: byte;
 begin
  Port[BaseAddress+Register_Port]:=DMA_Info;
  a:=Port[BaseAddress+Data_Port];
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.DMA     : 8:');
   case (a and $B) of
    1: Db.Writestring('0 - 16:');
    2: Db.Writestring('1 - 16:');
    8: Db.Writestring('3 - 16:');
   else
    Db.Writestring('? - 16:');
   end;
   case ((a and $E0) shr 4) of
    2: Db.Writelnstring('5.');
    4: Db.Writelnstring('6.');
    8: Db.Writelnstring('7.');
   else
    Db.Writelnstring('Unknown.');
   end;
  {$ENDIF}
  Get_DMA:=a;
 end;
{==========================================================================}
Function Get_IRQ_Status: byte;
 var a: byte;
 begin
  Port[BaseAddress+Register_Port]:=IRQ_Status;
  a:=Port[BaseAddress+Data_Port];
  {$IFDEF DEBUGINFO}
   Db.Writestring('MIXER.IRQ_Stat: 8: ');
   If (a and 1)=1 then Db.Writestring('Active - 16: ')
   else Db.Writestring('NonActive - 16: ');
   If (a and 2)=2 then Db.Writestring('Active - MPU: ')
   else Db.Writestring('NonActive - MPU: ');
   If (a and 4)=4 then Db.Writelnstring('Active.')
   else Db.Writelnstring('NonActive.');
  {$ENDIF}
  Get_IRQ_Status:=a and 3;
 end;
{==========================================================================}

BEGIN
END.