{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT PIC;
{ Unit that handles the Programmable Interrupt Controller (PIC).           }
{ Created          : 13/05/98                                              }
{ Last modification: 23/05/98                                              }

{{$DEFINE DEBUGINFO}

INTERFACE

{==========================================================================}
Procedure EnableChannel(Channel: byte);
{Enables the given IRQ channel.                                            }
Procedure DisableChannel(Channel: byte);
{Disables the given IRQ channel.                                           }
Procedure SendEndOfInterrupt(Channel: byte);
{Sends an end of interrupt to the PIC - This has to be done in the ISR.    }
{==========================================================================}

IMPLEMENTATION

{USES DB;}

CONST
     LowCmdStatus_reg   = $20;
     HighCmdStatus_reg  = $A0;
     LowMask_reg        = $21;
     HighMask_reg       = $A1;
     EndOfInterrupt     = $20;

{==========================================================================}
Procedure EnableChannel(Channel: byte);
 var OldValue: byte;
 begin
  {$IFDEF DEBUGINFO}
   DB.Writestring('PIC.Enable    : IRQ ');
   Db.Writebyte(Channel);
   Db.Writelnstring('.');
  {$ENDIF}
  if Channel<8 then OldValue:=Port[LowMask_reg]
               else OldValue:=Port[HighMask_reg];
  {$IFDEF DEBUGINFO}
   DB.Writestring('PIC.Enable    : OldValue ');
   Db.Writebyte(OldValue);
   Db.Writelnstring('.');
  {$ENDIF}
  if Channel<8 then Port[LowMask_reg]:=(OldValue and
                    ((1 shl Channel) xor $FF))
               else Port[HighMask_reg]:=(OldValue and
                    ((1 shl (Channel-8)) xor $FF));
 end;
{==========================================================================}
Procedure DisableChannel(Channel: byte);
 var OldValue: byte;
 begin
  {$IFDEF DEBUGINFO}
   DB.Writestring('PIC.Disable   : IRQ ');
   Db.Writebyte(Channel);
   Db.Writelnstring('.');
  {$ENDIF}
  if Channel<8 then OldValue:=Port[LowMask_reg]
               else OldValue:=Port[HighMask_reg];
  {$IFDEF DEBUGINFO}
   DB.Writestring('PIC.Disable   : OldValue ');
   Db.Writebyte(OldValue);
   Db.Writelnstring('.');
  {$ENDIF}
  if Channel<8 then Port[LowMask_reg]:=(OldValue or
                    (1 shl Channel))
               else Port[HighMask_reg]:=(OldValue or
                    (1 shl (Channel-8)));
 end;
{==========================================================================}
Procedure SendEndOfInterrupt(Channel: byte);
 begin
  If Channel<8 then Port[LowCmdStatus_reg]:=EndOfInterrupt
               else Port[HighCmdStatus_reg]:=EndOfInterrupt;
 end;
{==========================================================================}

BEGIN
END.