{**************************************************************************}
{This sourcecode is part of the program mixer347. If you have any questions}
{or want to use or modify any part of it please email me (jacmet@iname.com)}
{**************************************************************************}
UNIT STARTUP;
{ Statup/Initialization code.                                              }
{ Created          : 11/05/98                                              }
{ Last modification: 21/05/98                                              }

{{$DEFINE DEBUGINFO}

INTERFACE

{==========================================================================}
Procedure Init;
{initalizes system (soundcard,DMA,PIC,GUI), sets up an interrupthandler and}
{load in the filter koefficients.                                          }
{==========================================================================}

IMPLEMENTATION

USES VARIABLE,DSP,MIXER,DMA{,DB},DOS,PIC,GUI2,UTIL;

{==========================================================================}
{resets variables                                                          }
Procedure ResetVariables;
 var n: dword;
 begin
  for n:=0 to (2*BufferSize-1) do
   begin
    InputBuffer^[n,1]:=0;
    InputBuffer^[n,2]:=0;
    OutputBuffer^[n]:=0;
    X1[n]:=0;
    X2[n]:=0;
   end;

  Gate1Threshold:=0; Gate2Threshold:=0;
  Gate1Hysterisis:=0; Gate2Hysterisis:=0;
  Gate1HoldTime:=2205; Gate2HoldTime:=2205;
  Gate1ReleaseSpeed:=1; Gate2ReleaseSpeed:=1;
  Gate1AttackSpeed:=1; Gate2AttackSpeed:=1;
  Gate1HoldCount:=0; Gate2HoldCount:=0;
  Gate1Amp:=1; Gate2Amp:=1;

  A1lowpass:=1.0; A1bandpass:=1.0; A1highpass:=1.0;
  A2lowpass:=1.0; A2bandpass:=1.0; A2highpass:=1.0;
  Volume1:=1.0; Volume2:=1.0;
  Gain1:=1.0; Gain2:=1.0;
  MasterVolume:=1.0;
  CrossVolume:=1.0;
  Stopprogram:=false;
  RecalcImpulse1:=True; RecalcImpulse2:=True;

  Oldinputposition:=buffersize;
  OldOutputposition:=buffersize;
  inputposition:=buffersize;
  outputposition:=buffersize;

 end;
{==========================================================================}
{reads filter coefficients.                                                }
Procedure ReadCoefficients(filename: string);
 var f: file;
 begin
  If not(FileExist(filename)) then
   begin
    writeln('ERROR! file '+filename+' not found!');
    writeln('Halting..');
    halt;
   end;
  assign(f,filename);
  System.reset(f,1);
  blockread(f,Hlowpass,(FirOrder+1)*sizeof(Double));
  blockread(f,Hbandpass,(FirOrder+1)*sizeof(Double));
  blockread(f,Hhighpass,(FirOrder+1)*sizeof(Double));
  close(f);
 end;
{==========================================================================}
{Detect soundblaster and fills IRQ,DMA8 and DMA16                          }
Function DetectSoundBlaster16: boolean;
 var detected: boolean;
     a: byte;
 begin
  BaseAddress:=$200;
  repeat
   inc(BaseAddress,$20);
   {$IFDEF DEBUGINFO}
    Db.Writestring('SB.BaseAddress: ');
    Db.Writeword(word(BaseAddress));
    Db.Writelnstring('.');
   {$ENDIF}
   Detected:=DSP.Reset;
  until ((Detected) or (BaseAddress=$280));

  If ((Detected) and (DSP.GetVersion>=$400)) then
   begin
    DetectSoundBlaster16:=True;
    IRQ:=MIXER.Get_IRQ;
    a:=MIXER.Get_DMA;
    case (a and $B) of
     1: DMA8:=0;
     2: DMA8:=1;
     8: DMA8:=3;
    else
     DMA8:=$FF;
     DetectSoundblaster16:=False;
    end;
    case ((a and $E0) shr 4) of
     2: DMA16:=5;
     4: DMA16:=6;
     8: DMA16:=7;
    else
     DMA16:=$FF;
     DetectSoundblaster16:=False;
    end;
   end
  else
   begin
    BaseAddress:=$FFFFFFFF;
    IRQ:=$FF;
    DMA8:=$FF; DMA16:=$FF;
    DetectSoundBlaster16:=False;
   end;
 end;
{==========================================================================}
{Hooks interrupt                                                           }
Procedure HookInterrupt(IRQchannel: byte; NewInterruptVector: pointer);
 begin
  PIC.EnableChannel(IRQchannel);
  IF IRQchannel<8 then IRQchannel:=IRQchannel+8
     else IRQchannel:=IRQchannel+$70-8;

  DOS.GetIntVec(IRQchannel,Oldvector);
  DOS.SetIntvec(IRQchannel,NewInterruptVector);
 end;
{==========================================================================}
{interrupthandler                                                          }
procedure InterruptHandler; interrupt;
 var IRQtype: byte;
 begin
  IRQtype:=MIXER.Get_IRQ_status;

  if boolean(IRQtype and 1) then
   begin
    OutputPosition:=(Outputposition + Buffersize) and (2*Buffersize-1);
    DSP.SendEndOfInterrupt(DMA8);
   end;

  if boolean(IRQtype and 2) then
   begin
    InputPosition:=(Inputposition + Buffersize) and (2*Buffersize-1);
    DSP.SendEndOfInterrupt(DMA16);
   end;
  PIC.SendEndOfInterrupt(IRQ);
 end;
{==========================================================================}
{exitprocedure                                                             }
Procedure ExitProcedure;
 begin
  Exitproc:=OldExitprocedure;
  pause8bitDMA;
  Pause16bitDMA;
  GUI2.Deinit;
  IF IRQ<8 then Dos.SetIntVecFar(IRQ+8,Oldvector)
  else Dos.SetIntVecFar(IRQ+$70-8,Oldvector);
 end;
{==========================================================================}
Procedure Init;
 begin
  ReadCoefficients(Koefficientfilename);
  {Soundblaster 16 detection}
  If not(DetectSoundblaster16) then
   begin
    Writeln('Error! Soundblaster 16 not detected!');
    Writeln('Halting..');
    Halt;
   end
  else
   begin
    Writeln('Soundblaster 16 detected at IRQ ',IRQ,' DMA ',DMA8,' & ',DMA16,'.');
   end;
  {allocate dma-buffers}
  DMA.Allocatebuffer(Inputbuffer,sizeof(InputBuffer^));
  DMA.Allocatebuffer(Outputbuffer,sizeof(OutputBuffer^));
  if (dword(Inputbuffer)=0) or (dword(Outputbuffer)=0) then
   begin
    writeln('error allocation DMAbuffer');
    Writeln('Halting..');
    Halt;
   end;
  {reset variables}
  Resetvariables;
  {init GUI}
  GUI2.Init;
  {disable soundblaster mixer output}
  MIXER.Set_Output_Control(False,False,False,False,False);
  MIXER.Set_AutoGainCtrl(False);
  {install soundblaster IRQ-handler}
  HookInterrupt(IRQ,@InterruptHandler);
  {install exitprocedure}
  OldExitprocedure:=exitproc;
  exitproc:=@ExitProcedure;
  {set samplerate}
  DSP.Set_Samplerate(44100);
  {enable IRQ channel}
  PIC.EnableChannel(IRQ);
  {program dma-controller for 16bit recording}
  DMA.DisableChannel(DMA16);
  DMA.ProgramChannel(DMA16,AutoInitRecord,InputBuffer,(sizeof(InputBuffer^) div 2));
  {setup soundblaster for 16bit dma recording}
  DSP.SetupDMAmode(_16bit+Fifo+Auto_init+ADC,signed+stereo,BufferSize*2);
  {start it}
  DMA.EnableChannel(DMA16);
  {program dma-controller for 8bit playback}
  DMA.DisableChannel(DMA8);
  DMA.ProgramChannel(DMA8,AutoInitPlay,OutputBuffer,sizeof(OutputBuffer^));
  {setup soundblaster for 8bit dma playback}
  DSP.SetupDMAmode(_8bit+Fifo+Auto_init+DAC,signed+Mono,BufferSize);
  {start it}
  DMA.EnableChannel(DMA8);
 end;
{==========================================================================}

Begin
end.
